import React, { useEffect, useState, useCallback } from 'react';
import Info from '../../components/info';
import { addVisualization } from '../../data-access/store/visualizationSlice';
import { useAppDispatch, useCheckPermissionPolicy } from '../../data-access';
import { Visualizations } from '../components/VisualizationPanel';

type VisualizationDescription = {
  name: string;
  displayName: string;
  description: string;
};

export function Recommender() {
  const dispatch = useAppDispatch();
  const [visualizationDescriptions, setVisualizationDescriptions] = useState<VisualizationDescription[]>([]);

  const { canRead, canWrite } = useCheckPermissionPolicy();
  const [readAllowed, setReadAllowed] = useState(false);
  const [writeAllowed, setWriteAllowed] = useState(false);
  const resource = 'visualization';

  const checkReadPermission = useCallback(async () => {
    const result = await canRead(resource);
    setReadAllowed(result);
  }, [canRead]);

  const checkWritePermission = useCallback(async () => {
    const result = await canWrite(resource);
    setWriteAllowed(result);
  }, [canWrite]);

  useEffect(() => {
    checkReadPermission();
  }, [checkReadPermission]);

  useEffect(() => {
    checkWritePermission();
  }, [checkWritePermission]);

  useEffect(() => {
    const loadVisualizations = async () => {
      const descriptions = await Promise.all(
        Object.keys(Visualizations).map(async (key) => {
          const visualizationModule = await Visualizations[key]();
          const { default: visualizationComponent } = visualizationModule;
          return {
            name: key,
            displayName: visualizationComponent.displayName,
            description: visualizationComponent.description,
          };
        }),
      );
      setVisualizationDescriptions(descriptions);
    };

    loadVisualizations();
  }, []);

  return (
    <div className="p-4">
      <span className="text-md">Select a visualization</span>
      <div className="grid grid-cols-3 gap-4 my-2">
        {visualizationDescriptions.map(({ name, displayName, description }) => (
          <div
            key={name}
            className={`p-4 border ${writeAllowed ? 'cursor-pointer hover:bg-secondary-100' : 'cursor-not-allowed opacity-50'}`}
            onClick={async (e) => {
              e.preventDefault();
              if (!writeAllowed) {
                console.debug('User blocked from editing query due to being a viewer');
                return;
              }

              const component = await Visualizations[name]();
              dispatch(addVisualization({ ...component.default.settings, name: name, id: name }));
            }}
          >
            <div className="flex items-center justify-between">
              <span className="text-sm font-semibold truncate">{displayName}</span>
              <Info tooltip="Here an explanation" placement="top" />
            </div>
            <div className="flex items-center justify-between">
              <span className="text-xs text-secondary-400 truncate">{description}</span>
            </div>
          </div>
        ))}
      </div>
    </div>
  );
}
